/*
 * Run a program using the supplied arguments.
 *
 * Copyright (C) 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <stdio.h>
#include <sys/types.h>
#include <errno.h>

/* NOTE: To be provided via a header. */

extern pid_t popenv(int, const char *[], FILE **input, FILE **output, FILE **error);



/* Transfer size for communication. */

static const offset_t TO_TRANSFER = 1024;



/* List objects in the filesystem image. */

int main(int argc, char *argv[])
{
    FILE *input, *output, *error;
    pid_t pid;
    char buffer[TO_TRANSFER];
    size_t nread;

    if (argc < 2)
    {
        printf("Usage: test_popenv <program> <argument>...\n");
        return 1;
    }

    printf("Running: %s\n", argv[1]);

    pid = popenv(argc - 1, (const char **) argv + 1, &input, &output, &error);

    if (pid == -1)
    {
        printf("Error: %d\n", errno);
        return 1;
    }

    printf("Process: %d\n", pid);

    /* Wait for events on the streams and the process.
       NOTE: Currently just reading from the process. */

    while ((nread = fread(buffer, sizeof(char), TO_TRANSFER, output)))
    {
        if (!fwrite(buffer, sizeof(char), nread, stdout))
            break;
    }

    fclose(input);
    fclose(output);

    /* NOTE: Error not yet used. */

    return 0;
}

/* vim: tabstop=4 expandtab shiftwidth=4
*/
